/****************************************************************************
*																			*
* File    : hexview.c	 													*
*																			*
* Purpose : Sample add-in DLL.  	    										*
*																			*
* History : Date	   Reason												*
*			 04-01-10  Created												*
*																			*
*******************************************************************/

#define WIN32_LEAN_AND_MEAN 
#include <windows.h>
#include <windowsx.h>
#include <addin.h>
#include <stdlib.h>
#include "indent.h"
#include <commctrl.h>
#include <commdlg.h>
#include <tchar.h>  
//#include "..\dump.h"

// Private command ID - any number will do. 
#define ID_INDENT  1

static LRESULT CALLBACK MainWndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);


// Locals.
static HANDLE g_hmod = NULL;
static HWND g_hwndMain = NULL;
static HANDLE g_hDLL = NULL;
static HWND hDlgTree;

static HTREEITEM hRoot = 0, hItem = 0;
static int iLevelLocal = 0;

static LRESULT CALLBACK TreeDlgProc(HWND hdlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
static HTREEITEM treeAddItem(int iImage, int iSelectedImage, int iChar, int iLevel, char *lpszCode);
HANDLE listHandle(char* lpszAction,HANDLE handleDoc,HANDLE handleTree);

#define ITEMMAX 100000
#define LINELENMAX 1024
static char linebuffer[LINELENMAX];

typedef struct
{
	int dLine;
	char *pChar1;
	char *pChar2;
	char *szName;
} ITEM;
ITEM items[ITEMMAX + 1];

/****************************************************************************
*																			*
* Function: DllMain														*
*																			*
* Purpose : DLL entry and exit procedure.									*
*																			*
* History : Date	   Reason												*
*			 04-01-10  Created												*
*																			*
****************************************************************************/

BOOL WINAPI DllMain(HANDLE hDLL, DWORD dwReason, LPVOID lpReserved)
{
	switch (dwReason)
	{
		case DLL_PROCESS_ATTACH:
			g_hDLL = g_hmod = hDLL;
			return TRUE;
			
		case DLL_PROCESS_DETACH:
			return TRUE;
			
		default:
			return TRUE;
	}
}

/****************************************************************************
*																			*
* Function: AddInMain														*
*																			*
* Purpose : Add-in entry and exit procedure.								*
*																			*
* History : Date	   Reason												*
*			 04-01-10  Created												*
*																			*
****************************************************************************/

ADDINAPI BOOL WINAPI AddInMain(HWND hwnd, ADDIN_EVENT eEvent)
{
	switch (eEvent)
	{
		case AIE_APP_CREATE:
			{
				ADDIN_ADD_COMMAND AddCmd =
				{ 0
				};
				
				
				/* Save handle of main IDE window */
				g_hwndMain = hwnd;
				
				/* Add command button to IDE toolbar */
				AddCmd.cbSize = sizeof(AddCmd);
				AddCmd.pszText = "indent the active document";	// Tooltip text.
				AddCmd.hIcon = LoadImage(g_hmod, MAKEINTRESOURCE(IDR_ICON1), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR | LR_SHARED);
				AddCmd.id = ID_INDENT;
				//AddCmd.id=AIM_MENU_NOTHING;
				
				return AddIn_AddCommand(hwnd, &AddCmd);
			}
			
		case AIE_APP_DESTROY:
			/* Remove button from IDE toolbar */
			return AddIn_RemoveCommand(hwnd, ID_INDENT);
			
		default:
			return TRUE;
	}
}

/****************************************************************************
*																			*
* Function: AddInCommand													*
*																			*
* Purpose : Add-in command handler.										*
*																			*
* History : Date	   Reason												*
*			 04-01-10  Created												*
*																			*
****************************************************************************/

ADDINAPI void WINAPI AddInCommand(int idCmd)
{
	switch (idCmd)
	{
		case ID_INDENT:
			{
				
				HWND hwndDoc = AddIn_GetActiveDocument(g_hwndMain);
				if(AID_SOURCE!=AddIn_GetDocumentType(hwndDoc))
					return;
				InitCommonControls();
				
				BOOL isComment1 = FALSE;	//........//xyz..........
				BOOL isComment2 = FALSE;	//........./*xyz*/......
				BOOL isString1 = FALSE;	//.........'x'........
				BOOL isString2 = FALSE;	//........."xyz"........
				BOOL isBackSlash = FALSE;
				BOOL lastcolon = FALSE;
				char szFilename[1000];
				
				
				int iForWhileIf[1024];	// used for the levels of nested "if, for ,while"
				memset(iForWhileIf, 0, sizeof(iForWhileIf));
				
				int iSwitch[1024];	// used for the levels of nested "switch, case"
				memset(iSwitch, 0, sizeof(iSwitch));
				
				
				int lenNew, len = AddIn_GetSourceTextLength(hwndDoc) + 1;
				ADDIN_RANGE addin_range=
				{0
				};
				
				ADDIN_DOCUMENT_INFO addin_di =
				{ 0
				};
				addin_di.cbSize = sizeof(addin_di);
				AddIn_GetDocumentInfo(hwndDoc, &addin_di);
				GetFileTitle(addin_di.szFilename, szFilename, sizeof(szFilename));
				
				//malloc original source
				char *lpszText = (char *)malloc(len);
				if (lpszText == NULL)
				{
					MessageBox(hwndDoc, "Error with malloc Text", "indent error", MB_OK | MB_ICONEXCLAMATION | MB_TASKMODAL | MB_TOPMOST);
					break;
				}
				
				//malloc indented source
				char *lpszTextNew = (char *)malloc(lenNew = len * 10 + 1000);	//I "hope" it is enough...
				if (lpszText == NULL)
				{
					MessageBox(hwndDoc, "Error with malloc TextNew", "indent error", MB_OK | MB_ICONEXCLAMATION | MB_TASKMODAL | MB_TOPMOST);
					free(lpszText);
					break;
				}
				
				if(listHandle("is",hwndDoc,NULL) )	// if codetree window already exists
				{
					/*hDlgTree=listHandle("getTree",hwndDoc,NULL);
					ShowWindow(hDlgTree,SW_RESTORE);
					TreeView_DeleteAllItems(GetDlgItem(hDlgTree, IDTREE));*/
					DestroyWindow(hDlgTree);
				}
				{
					hDlgTree = CreateDialog(g_hDLL, MAKEINTRESOURCE(DLG_TREE),hwndDoc ,(DLGPROC)TreeDlgProc);
					
					RECT rcDoc;
					GetClientRect(hwndDoc,&rcDoc);
					MoveWindow(hDlgTree,
						(rcDoc.right-rcDoc.left)/2,
						0,
						(rcDoc.right-rcDoc.left)/2,
						rcDoc.bottom-rcDoc.top,
						TRUE);
					SetWindowText(hDlgTree , szFilename);
					SetWindowLong(hDlgTree , DWL_USER, (LONG)hwndDoc);
					
					listHandle("add",hwndDoc,hDlgTree);
				}
				
				
				// prepare indenting
				AddIn_GetSourceText(hwndDoc, lpszText, len);
				AddIn_GetSourceSel(hwndDoc, &addin_range);
				
				char *lpszTextNewEnd = lpszTextNew + lenNew - 3;
				
				char *lpszC = lpszText;	//original source
				char *lpszCNew = lpszTextNew;	// indented source
				
				int offset = 0;	/*  { ... }level  */
				int offset3 = 0;	/*  ( ... )level  */
				
				
				int iSelectedSourceToIndented=0;
				/**** all indenting is done in one loop ! ****/
				while (*lpszC != '\0' && lpszCNew < lpszTextNewEnd)
				{
					if(lpszC-lpszText>=addin_range.iStartPos&&lpszC-lpszText<=addin_range.iEndPos)
						iSelectedSourceToIndented=lpszCNew-lpszTextNew;
					
					
					/***** strings and comments identifidation ****/
					
					if ((*lpszC == '/') && (*(lpszC + 1) == '*') && (isComment1 == FALSE) && (isComment2 == FALSE) && (isString1 == FALSE) && (isString2 == FALSE))
						isComment2 = TRUE;
					if ((*lpszC == '*') && (*(lpszC + 1) == '/') && (isComment1 == FALSE) && (isComment2 == TRUE) && (isString1 == FALSE) && (isString2 == FALSE))
						isComment2 = FALSE;
					
					if ((*lpszC == '/') && (*(lpszC + 1) == '/') && (isComment2 == FALSE) && (isString1 == FALSE) && (isString2 == FALSE))
						isComment1 = TRUE;
					
					if ((*lpszC == '"') && (isString1 == FALSE) && (isString2 == FALSE) && (isComment1 == FALSE) && (isComment2 == FALSE))
						isString2 = TRUE;
					else if ((isString2 == TRUE) && (*lpszC == '"') && (!isBackSlash) && (isString1 == FALSE) && (isComment1 == FALSE) && (isComment2 == FALSE))
						isString2 = FALSE;
					
					if ((*lpszC == '\'') && (isString1 == FALSE) && (isString2 == FALSE) && (isComment1 == FALSE) && (isComment2 == FALSE))
						isString1 = TRUE;
					else if ((isString1 == TRUE) && (*lpszC == '\'') && (!isBackSlash) && (isString2 == FALSE) && (isComment1 == FALSE) && (isComment2 == FALSE))
						isString1 = FALSE;
					
					
					if ((isString1 || isString2) && (*lpszC == '\\') && isBackSlash == FALSE)
						isBackSlash = TRUE;
					else
						isBackSlash = FALSE;
					
					/*****END OF strings and comments identifidation ****/
					
					// jump over a backslash that connects a line
					if ((*lpszC == '\\') && (*(lpszC + 1) == '\n' || *(lpszC + 1) == '\r'))
						*(lpszCNew++) = *(lpszC++);
					
					/***** Calculating the indenting *****/
					if ((!isString1) && (!isString2) && (!isComment1) && (!isComment2))
					{
						
						/***** (   )****/
						if (*lpszC == '(')
						{
							offset3++;
						}
						if (*lpszC == ')')
							offset3--;
						if (offset3 < 0)
							offset3 = 0;
						/*****END OF (   )****/
						
						
						/***** nestet for,while, if****/
						if (*lpszC == ';')
						{
							while (iForWhileIf[offset + offset3] == 4)
							{
								iForWhileIf[offset + offset3] = 0;
								offset--;
								if (offset < 0)
									offset = 0;
							}
						}
						
						if (iForWhileIf[offset + offset3] == 3)
						{
							if (*lpszC == '{' || *lpszC == ';')
							{
								iForWhileIf[offset + offset3] = 0;
								offset--;
								if (*(lpszCNew - 1) == '\t')
									lpszCNew--;
							}
							else if (isgraph(*lpszC))
							{
								iForWhileIf[offset + offset3] = 4;
							}
						}
						
						if (iForWhileIf[offset + offset3 + 1] == 2 && *lpszC == ')')
						{
							iForWhileIf[offset + offset3 + 1] = 3;
							offset++;
						}
						if (iForWhileIf[offset + offset3] == 1 && *lpszC == '(')
						{
							iForWhileIf[offset + offset3] = 2;
						}
						else if (iForWhileIf[offset + offset3 + 1] == 1 && isgraph(*lpszC))
						{
							iForWhileIf[offset + offset3 + 1] = 0;
						}
						if (0 == strncmp(lpszC, "for", 3))
						{
							strncpy(lpszCNew, lpszC, 3);
							lpszCNew += 2;
							lpszC += 2;
							iForWhileIf[offset + offset3 + 1] = 1;
						}
						else if (0 == strncmp(lpszC, "while", 5))
						{
							strncpy(lpszCNew, lpszC, 5);
							lpszCNew += 4;
							lpszC += 4;
							iForWhileIf[offset + offset3 + 1] = 1;
						}  
						else if (0 == strncmp(lpszC, "else if", 7))
						{
							strncpy(lpszCNew, lpszC, 7);
							lpszCNew += 6;
							lpszC += 6;
							iForWhileIf[offset + offset3 + 1] = 1;
						}
						else if (0 == strncmp(lpszC, "if", 2))
						{
							strncpy(lpszCNew, lpszC, 2);
							lpszCNew += 1;
							lpszC += 1;
							iForWhileIf[offset + offset3 + 1] = 1;
						}
						else if (0 == strncmp(lpszC, "else", 4))
						{
							strncpy(lpszCNew, lpszC, 4);
							lpszCNew += 3;
							lpszC += 3;
							iForWhileIf[offset + offset3 + 1] = 3;
							offset++;
						}
						
						
						/*****END OF  nestet for,while, if****/
						
						
						/***** swith case default ****/
						else if (0 == strncmp(lpszC, "switch", 6))
						{
							strncpy(lpszCNew, lpszC, 6);
							lpszCNew += 5;
							lpszC += 5;
						}
						
						else if ((0 == strncmp(lpszC, "case", 4) && isspace(*(lpszC + 4))) || (0 == strncmp(lpszC, "default:", 8) && isspace(*(lpszC + 8))))
						{
							if (*(lpszCNew - 1) == '\t' && iSwitch[offset] == 1)
								lpszCNew--;
							iSwitch[offset] = 1;
						}
						/*****END OF  swith case default****/
						
						
						if (*lpszC == '"')
							lastcolon = TRUE;
						else if isgraph
						(*lpszC)lastcolon = FALSE;
						
						/***** {   }****/
						if (*lpszC == '{')
						{
							char *lpszTest = lpszCNew - 1;
							while (lpszTest > lpszTextNew && !isgraph(*lpszTest))
								lpszTest--;
							lpszCNew = lpszTest + 1;
							*lpszCNew++ = '\n';
							int iSwitchSum = 0;
							for (int i = 0; i <= offset + offset3; i++)
								iSwitchSum += iSwitch[i];
							for (int n = 0; n < offset + offset3 + iSwitchSum; n++)
							{
								*(lpszCNew++) = '\t';
							}
							
							offset++;
						}
						if (*lpszC == '}')
						{
							char *lpszTest = lpszCNew - 1;
							while (lpszTest > lpszTextNew && !isgraph(*lpszTest))
								lpszTest--;
							lpszCNew = lpszTest + 1;
							*lpszCNew++ = '\n';
							int iSwitchSum = 0;
							for (int i = 0; i <= offset + offset3; i++)
								iSwitchSum += iSwitch[i];
							for (int n = 0; n < offset + offset3 + iSwitchSum - 1 - iSwitch[offset]; n++)
							{
								*(lpszCNew++) = '\t';
							}
							
							offset--;
							iSwitch[offset + 1] = 0;
						}
						/***** {   }****/
						
						if (offset < 0)
							offset = 0;
					}
					
					/*****END OF Calculating the indenting *****/
					
					
					
					/***** indenting by adding tabs at the beginning of the lines****/
					
					if (*lpszC == '\n')
					{
						int offsetlastcolon = 0;	// indents strings that are more than one line
						if (lastcolon)
							offsetlastcolon = 1;
						else
							offsetlastcolon = 0;
						if (isComment2 == FALSE)
						{
							isComment1 = FALSE;
							isString1 = FALSE;
							isString2 = FALSE;
						}
						*(lpszCNew++) = *(lpszC++);
						while (isspace(*lpszC) && (*lpszC != '\r') && (*lpszC != '\n'))
							lpszC++;
						if (*lpszC != '"')
							offsetlastcolon = 0;
						int iSwitchSum = 0;
						for (int i = 0; i <= offset + offset3; i++)
							iSwitchSum += iSwitch[i];
						for (int n = 0; n < offset + offset3 + offsetlastcolon + iSwitchSum; n++)
						{
							*(lpszCNew++) = '\t';	// here is the indenting !
						}
					}
					/*****END OF indenting by adding tabs at the beginning of the lines****/
					
					else
					{
						*(lpszCNew++) = *(lpszC++);
					}
				}
				*lpszCNew = '\0';
				/****END OF all indenting is done in one loop ! ****/
				
				/***The result of the Indenting is written to your Source File. ***/
				if (lpszCNew < lpszTextNewEnd)
				{
					
					AddIn_SetSourceText(hwndDoc, lpszTextNew);
					//			AddIn_SetSourceSel(hwndDoc, &addin_range);
					//			AddIn_ScrollSourceCaret(hwndDoc);
				}
				/***END OF   The result of the Indenting is written to your Source File. ***/
				else
				{
					MessageBox(hwndDoc, "TextNew was too small", "indent error", MB_OK | MB_ICONEXCLAMATION | MB_TASKMODAL | MB_TOPMOST);
				}
				
				
				/***** Feed the Tree  (number of indenting tabs determins the level of the item in the tree structure) *****/
				
				hRoot = 0;
				hItem = 0;
				iLevelLocal = 0;
				
				lpszCNew = lpszTextNew;
				
				int iItem=1;
				HTREEITEM hSelectedItem=0;
				HTREEITEM hSelectedItemTemp=0;
				
				
				treeAddItem(1, 1, 0, 0, lpszCNew);// the first line of the source does not begin with a linefeed.
				
				while (*lpszCNew != '\0' && lpszCNew < lpszTextNewEnd)
				{
					if (lpszCNew-lpszTextNew ==  iSelectedSourceToIndented)
					{
						hSelectedItem=hSelectedItemTemp;
					}
					if (*lpszCNew++ == '\n')
					{
						char *lpszTest = lpszCNew;
						int iTab = 0;
						while (*lpszTest++ == '\t')
							iTab++;
						hSelectedItemTemp=treeAddItem(1, 1, lpszCNew - lpszTextNew, iTab, lpszCNew);
						iItem++;
					}
				}
				/*****END OF  Feed the Tree  (number of indenting tabs determins the level of the item in the tree structure) *****/
				
				TreeView_Select(GetDlgItem(hDlgTree, IDTREE),hSelectedItem,TVGN_CARET	);
				//				TreeView_Select(GetDlgItem(hDlgTree, IDTREE),hSelectedItem,TVGN_FIRSTVISIBLE	);
				TreeView_Select(GetDlgItem(hDlgTree, IDTREE),hSelectedItem,TVGN_DROPHILITE
					);
				
				free(lpszText);
				free(lpszTextNew);
				break;
			}
	}
}
static LRESULT CALLBACK TreeDlgProc(HWND hdlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	HWND hwndDoc = (HWND)GetWindowLong(hdlg, DWL_USER);
	
	switch (uMsg)
	{
		case WM_COMMAND:
			switch LOWORD
			(wParam)
			{
				case IDOK:
					{
						return TRUE;
					}
				case IDCANCEL:
					{
						EndDialog(hdlg, 0);
						return TRUE;
					}
			}
			return FALSE;
			
		case WM_NOTIFY:
			{
				NMHDR *pnmh = (LPNMHDR)lParam;
				//  if(pnmh->hwndFrom==  )
				switch (pnmh->idFrom)
				{
					case IDTREE:
						switch (pnmh->code)
						{
							case TVN_SELCHANGED:
								{
									
									SHORT isCtrl= GetAsyncKeyState(VK_CONTROL)&0x8000;
									
									static int iCharSource_Old=0;
									static int iCharEndLine_Old=0;
									ADDIN_RANGE aRange=
									{0,0
									};
									
									HTREEITEM hItem = TreeView_GetSelection(pnmh->hwndFrom);
									TV_ITEM tvItem =
									{ 0
									};									
									tvItem.mask = TVIF_PARAM || TVIF_TEXT;
									tvItem.hItem = hItem;
									tvItem.pszText=linebuffer;
									tvItem.cchTextMax=LINELENMAX;
									TreeView_GetItem(pnmh->hwndFrom, &tvItem);
									TreeView_Select(pnmh->hwndFrom,tvItem.hItem,TVGN_DROPHILITE);
									
									int iCharSource = tvItem.lParam;
									int iCharEndLine =iCharSource+strlen(linebuffer);
									if(! isCtrl)
									{
										aRange.iStartPos =iCharSource;
										aRange.iEndPos =iCharEndLine;
									}
									
									else
									{
										if(iCharSource>iCharSource_Old)
										{
											aRange.iStartPos =iCharSource_Old;
											aRange.iEndPos =iCharEndLine;
										}
										else
										{
											aRange.iStartPos =iCharSource;
											aRange.iEndPos =iCharEndLine_Old;
										}
									}
									AddIn_SetSourceSel(hwndDoc, &aRange);
									AddIn_ScrollSourceCaret(hwndDoc);
									AddIn_SetSourceSel(hwndDoc, &aRange);
									
									iCharSource_Old=iCharSource;
									iCharEndLine_Old=iCharEndLine;
								}
								break;
						}
						break;
				}
			}
			return FALSE;
			
		case WM_CLOSE:
			{	
				listHandle("destroy",hwndDoc,NULL);
				DestroyWindow(hdlg);
			}
			return FALSE;
			
		case WM_SIZE:
			{
				HWND hwndTreeView=GetDlgItem(hdlg,IDTREE);
				RECT rcDialog;
				GetClientRect(hdlg,&rcDialog);
				MoveWindow(hwndTreeView,
					0,
					0,
					rcDialog.right-rcDialog.left,
					rcDialog.bottom-rcDialog.top,
					TRUE);
			}
			return FALSE;
	}
	return FALSE;
}


static HTREEITEM treeAddItem(int iImage, int iSelectedImage, int iChar, int iLevel, char *lpszCode)
{
	linebuffer[0] = 0;
	
	char *lpszLine = &linebuffer[0];
	char *lpszMax = lpszCode + LINELENMAX - 1;
	char *lpszC = lpszCode;
	
	TV_INSERTSTRUCT tvins;
	
	if (iLevel == iLevelLocal + 1)
	{
		hRoot = hItem;
		iLevelLocal = iLevel;
	}
	else if (iLevel < iLevelLocal)
	{
		
		while (iLevel < iLevelLocal)
		{
			hRoot = TreeView_GetParent(GetDlgItem(hDlgTree, IDTREE), hRoot);
			iLevelLocal--;
		}
	}
	else if (iLevel == iLevelLocal)
	{
		// hRoot is unganged
		// iLevelLocal is unganged
	}
	
	/**** replace not printable characters by spaces ****/
	while (lpszC < lpszMax && *lpszC != '\n')
	{
		if (*lpszC == '\n')
			*lpszLine = ' ';
		else
		{
			*lpszLine = *lpszC;
		}
		if (!isgraph(*lpszC))
		{
			*lpszLine = ' ';
		}
		lpszC++;
		lpszLine++;
	}
	/****END OF replace not printable characters by spaces ****/
	
	
	
	
	*lpszLine = 0;
	ZeroMemory(&tvins, sizeof(tvins));
	tvins.item.mask = TVIF_TEXT | TVIF_PARAM;	// |TVIF_IMAGE|TVIF_SELECTEDIMAGE;
	tvins.item.lParam = iChar;
	tvins.item.cchTextMax = LINELENMAX;
	tvins.hParent = hRoot;
	//tvins.item.iImage=2;
	//tvins.item.iSelectedImage=3;
	tvins.item.pszText = &linebuffer[0];
	
	hItem = TreeView_InsertItem(GetDlgItem(hDlgTree, IDTREE), &tvins);
	return hItem;
}


// remember handles of the ide document windows with a codetree child window.
HANDLE listHandle(char* lpszAction,HANDLE handleDoc,HANDLE handleTree)
{
	
	
	struct element
	{
		struct element* lpNextElement;
		HANDLE handleDoc;
		HANDLE handleTree;
	};
	static struct element* lpFirstElement=NULL;
	struct element* lpElementTemp=NULL;
	int iFound=0;
	
	if(0==strcmp("add",lpszAction))
	{
		lpElementTemp=malloc(sizeof(struct element));
		if(lpElementTemp)
		{
			lpElementTemp->lpNextElement=lpFirstElement;
			lpElementTemp->handleDoc=handleDoc;
			lpElementTemp->handleTree=handleTree;
			lpFirstElement=lpElementTemp;
		}
		else MessageBox(0,"malloc failed in function listHandle","codetree",0);
	}
	else if(0==strcmp("is",lpszAction))
	{
		lpElementTemp=lpFirstElement;
		iFound=0;
		while((int)(lpElementTemp)&&!iFound)
		{
			if(lpElementTemp->handleDoc==handleDoc)iFound=1;
			lpElementTemp=lpElementTemp->lpNextElement;
		}
		if(!iFound)return NULL;
	}
	else if(0==strcmp("destroy",lpszAction))
	{
		lpElementTemp=lpFirstElement;
		iFound=0;
		if(! lpElementTemp)
			return NULL;
		struct element** lppPreviousPointer=NULL;
		while((NULL != lpElementTemp)&&!iFound)
		{
			if(lpElementTemp->handleDoc==handleDoc)
			{
				iFound=1;
				
				if(NULL != lppPreviousPointer)
					*lppPreviousPointer=lpElementTemp->lpNextElement;
				else 
					lpFirstElement=lpElementTemp->lpNextElement;
				free(lpElementTemp);
				return NULL;
			}
			lppPreviousPointer=&lpElementTemp->lpNextElement;
			lpElementTemp=lpElementTemp->lpNextElement;
		}
		if(!iFound)return NULL;
	}
	else if(0==strcmp("getTree",lpszAction))
	{
		lpElementTemp=lpFirstElement;
		while((int)(lpElementTemp)&&!iFound)
		{
			if(lpElementTemp->handleDoc==handleDoc)return lpElementTemp->handleTree;
			lpElementTemp=lpElementTemp->lpNextElement;
		}
		return NULL;
	}
	else MessageBox(0,"Wrong string parameter in listHandle","codetree",0);
	lpElementTemp=lpFirstElement;
	/*while(lpElementTemp)
	{
	
	sprintf(string,"%d",lpElementTemp->handle);
	MessageBox(0,string,"",0);
	lpElementTemp=lpElementTemp->lpNextElement;
	}*/
	return handleDoc;
}




